"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseTimeDimension = void 0;
const moment_timezone_1 = __importDefault(require("moment-timezone"));
const shared_1 = require("@cubejs-backend/shared");
const BaseFilter_1 = require("./BaseFilter");
const UserError_1 = require("../compiler/UserError");
const Granularity_1 = require("./Granularity");
class BaseTimeDimension extends BaseFilter_1.BaseFilter {
    dateRange;
    granularityObj;
    boundaryDateRange;
    shiftInterval;
    constructor(query, timeDimension) {
        super(query, {
            dimension: timeDimension.dimension,
            operator: 'in_date_range',
            values: timeDimension.dateRange
        });
        this.dateRange = timeDimension.dateRange;
        if (timeDimension.granularity) {
            this.granularityObj = new Granularity_1.Granularity(query, timeDimension);
        }
        this.boundaryDateRange = timeDimension.boundaryDateRange;
        this.shiftInterval = timeDimension.shiftInterval;
    }
    // TODO: find and fix all hidden references to granularity to rely on granularityObj instead?
    get granularity() {
        return this.granularityObj?.granularity;
    }
    selectColumns() {
        if (!this.granularityObj) {
            return null;
        }
        return super.selectColumns();
    }
    hasNoRemapping() {
        if (!this.granularityObj) {
            return false;
        }
        return super.hasNoRemapping();
    }
    aliasName() {
        if (!this.granularityObj) {
            return null;
        }
        return super.aliasName();
    }
    unescapedAliasName(granularity) {
        const actualGranularity = granularity || this.granularityObj?.granularity || 'day';
        const fullName = `${this.dimension}.${actualGranularity}`;
        if (this.query.options.memberToAlias?.[fullName]) {
            return this.query.options.memberToAlias[fullName];
        }
        return `${this.query.aliasName(this.dimension)}_${actualGranularity}`; // TODO date here for rollups
    }
    dateSeriesAliasName() {
        return this.query.escapeColumnName(`${this.dimension}_series`);
    }
    dateSeriesSelectColumn(dateSeriesAliasName, dateSeriesGranularity) {
        if (!this.granularityObj) {
            return null;
        }
        // In case of query with more than one granularity, the time series table was generated
        // with the minimal granularity among all. If this is our granularity, we can save
        // some cpu cycles without 'date_from' truncation. But if this is not our granularity,
        // we need to truncate it to desired.
        if (dateSeriesGranularity && this.granularityObj?.granularity !== dateSeriesGranularity) {
            return `${this.query.dimensionTimeGroupedColumn(`${dateSeriesAliasName || this.dateSeriesAliasName()}.${this.query.escapeColumnName('date_from')}`, this.granularityObj)} ${this.aliasName()}`;
        }
        return `${dateSeriesAliasName || this.dateSeriesAliasName()}.${this.query.escapeColumnName('date_from')} ${this.aliasName()}`;
    }
    dimensionSql() {
        const context = this.query.safeEvaluateSymbolContext();
        const granularityName = context.granularityOverride || this.granularityObj?.granularity;
        const path = granularityName ? `${this.expressionPath()}.${granularityName}` : this.expressionPath();
        const granularity = granularityName && this.granularityObj?.granularity !== granularityName ?
            new Granularity_1.Granularity(this.query, {
                dimension: this.dimension,
                granularity: granularityName
            }) : this.granularityObj;
        if (context.renderedReference?.[path]) {
            return context.renderedReference[path];
        }
        if (context.rollupQuery || context.wrapQuery) {
            if (context.rollupGranularity === this.granularityObj?.granularity) {
                return super.dimensionSql();
            }
            return this.query.dimensionTimeGroupedColumn(this.query.dimensionSql(this), granularity);
        }
        if (context.ungrouped) {
            return this.convertedToTz();
        }
        return this.query.dimensionTimeGroupedColumn(this.convertedToTz(), granularity);
    }
    dimensionDefinition() {
        if (this.shiftInterval) {
            return { ...super.dimensionDefinition(), shiftInterval: this.shiftInterval };
        }
        return super.dimensionDefinition();
    }
    convertTzForRawTimeDimensionIfNeeded(sql) {
        return sql();
    }
    convertedToTz() {
        return this.query.convertTz(this.query.dimensionSql(this));
    }
    filterToWhere() {
        if (!this.dateRange) {
            return null;
        }
        return super.filterToWhere();
    }
    filterParams() {
        if (!this.dateRange) {
            return [];
        }
        return super.filterParams();
    }
    dateFromFormattedValue = null;
    dateFromFormatted() {
        if (!this.dateFromFormattedValue) {
            this.dateFromFormattedValue = this.formatFromDate(this.dateRange[0]);
        }
        return this.dateFromFormattedValue;
    }
    dateFromValue = null;
    dateFrom() {
        if (!this.dateFromValue) {
            this.dateFromValue = this.inDbTimeZoneDateFrom(this.dateRange[0]);
        }
        return this.dateFromValue;
    }
    dateFromParam() {
        return this.query.paramAllocator.allocateParamsForQuestionString(this.query.timeStampParam(this), [this.dateFrom()]);
    }
    localDateTimeFromParam() {
        return this.query.dateTimeCast(this.query.paramAllocator.allocateParam(this.dateFromFormatted()));
    }
    localDateTimeFromOrBuildRangeParam() {
        return this.query.dateTimeCast(this.query.paramAllocator.allocateParam(this.dateRange ? this.dateFromFormatted() : shared_1.BUILD_RANGE_START_LOCAL));
    }
    dateToFormattedValue = null;
    dateToFormatted() {
        if (!this.dateToFormattedValue) {
            this.dateToFormattedValue = this.formatToDate(this.dateRange[1]);
        }
        return this.dateToFormattedValue;
    }
    dateToValue = null;
    dateTo() {
        if (!this.dateToValue) {
            this.dateToValue = this.inDbTimeZoneDateTo(this.dateRange[1]);
        }
        return this.dateToValue;
    }
    dateToParam() {
        return this.query.paramAllocator.allocateParamsForQuestionString(this.query.timeStampParam(this), [this.dateTo()]);
    }
    localDateTimeToParam() {
        return this.query.dateTimeCast(this.query.paramAllocator.allocateParam(this.dateToFormatted()));
    }
    localDateTimeToOrBuildRangeParam() {
        return this.query.dateTimeCast(this.query.paramAllocator.allocateParam(this.dateRange ? this.dateToFormatted() : shared_1.BUILD_RANGE_END_LOCAL));
    }
    dateRangeGranularity() {
        if (!this.dateRange) {
            return null;
        }
        const msFrom = moment_timezone_1.default.tz(this.dateFromFormatted(), this.query.timezone);
        const msTo = moment_timezone_1.default.tz(this.dateToFormatted(), this.query.timezone).add(1, 'ms');
        return this.query.minGranularity(this.query.granularityFor(msFrom), this.query.granularityFor(msTo));
    }
    rollupGranularityValue = null;
    rollupGranularity() {
        if (!this.rollupGranularityValue) {
            this.rollupGranularityValue =
                this.query.cacheValue(['rollupGranularity', this.granularityObj?.granularity].concat(this.dateRange), () => {
                    if (!this.granularityObj) {
                        return this.dateRangeGranularity();
                    }
                    // If we have granularity and date range, we need to check
                    // that the interval and the granularity offset are stacked/fits with date range
                    if (this.dateRange && (this.granularityObj.isPredefined() ||
                        !this.granularityObj.isAlignedWithDateRange([this.dateFromFormatted(), this.dateToFormatted()]))) {
                        return this.query.minGranularity(this.granularityObj.minGranularity(), this.dateRangeGranularity());
                    }
                    // We return the granularity as-is, including custom ones,
                    // because baseQuery.granularityHierarchies correctly expands all custom granularities into hierarchies.
                    return this.granularityObj.granularity;
                });
        }
        return this.rollupGranularityValue;
    }
    timeSeries() {
        if (!this.dateRange) {
            throw new UserError_1.UserError('Time series queries without dateRange aren\'t supported');
        }
        if (!this.granularityObj) {
            return [[this.dateFromFormatted(), this.dateToFormatted()]];
        }
        return this.granularityObj.timeSeriesForInterval([this.dateFromFormatted(), this.dateToFormatted()], { timestampPrecision: this.query.timestampPrecision() });
    }
    resolvedGranularity() {
        return this.granularityObj ? this.granularityObj.resolvedGranularity() : null;
    }
    wildcardRange() {
        return [shared_1.FROM_PARTITION_RANGE, shared_1.TO_PARTITION_RANGE];
    }
    boundaryDateRangeFormatted() {
        // TODO or here is due to boundaryDateRange can be defined in originalSql query used by rollup
        // TODO and dateRange can be defined in rollup query
        return this.boundaryDateRange && [
            this.formatFromDate(this.boundaryDateRange[0]),
            this.formatToDate(this.boundaryDateRange[1])
        ] || [this.dateFromFormatted(), this.dateToFormatted()];
    }
}
exports.BaseTimeDimension = BaseTimeDimension;
//# sourceMappingURL=BaseTimeDimension.js.map